#include <iostream>
#include <signal.h>
#include <Ice/Ice.h>
#include <Udi2_ice.h>

// ======================================================
// IMPLEMENTATION
// ======================================================
class UdiUser_impl : public MUdi::IUdiUser
{
    public:
        UdiUser_impl() {}
        std::string doOperation( std::string const & p_operationID, std::string const & p_params, Ice::Current const & p_iceCurrentNotUsed )
        {
            if ( p_operationID == "OK" )
            {
                std::cout << "Returning 'OK'\n" ;
                return "OK" ;
            }
            std::cout << "Throwing exception\n" ;
            throw ouMExceptions::MsgException2( 1, "ERROR" ) ;
            return "" ; // Will never reach this
        }
} ;

class Server : public Ice::Application
{
    public:

        Server() : Ice::Application() {}

        virtual int run( int argc, char* argv[] );
        virtual void interruptCallback( int p_signal )
        {
            communicator()->shutdown() ;
        }
};

/*-------------------------------------------------------------
 MAIN
 -------------------------------------------------------------*/
int main( int argc, char* argv[] )
{
    Server app ;
    return app.main( argc, argv );
}

int Server::run( int argc, char* argv[] )
{
    try
    {
        Ice::ObjectAdapterPtr adapter = communicator()->createObjectAdapterWithEndpoints( "UDI_SERVANT", "tcp -p 10001" ) ;
        Ice::Identity              id = communicator()->stringToIdentity( "UDI_OPERATION" );

        Ice::ObjectPtr object = new UdiUser_impl() ;
        adapter->add ( object, id ) ;
        adapter->activate();

        Ice::Application::callbackOnInterrupt() ;

        communicator()->waitForShutdown();
    }
    catch( Ice::Exception const & e )
    {
        std::cout << e.what() << std::endl ;
    }
    catch ( std::exception const & e )
    {
        std::cout << e.what() << std::endl ;
    }
    catch ( ... )
    {
        std::cout << "Unknown exception\n" ;
    }
    return 0 ;
}

